/****************************************************************************

  (c) SYSTEC electronic GmbH, D-08468 Heinsdorfergrund, Am Windrad 2
      www.systec-electronic.com

  Project:      asciiparse library

  Description:  implementation of library

  License:

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

    3. Neither the name of SYSTEC electronic GmbH nor the names of its
       contributors may be used to endorse or promote products derived
       from this software without prior written permission. For written
       permission, please contact info@systec-electronic.com.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.

    Severability Clause:

        If a provision of this License is or becomes illegal, invalid or
        unenforceable in any jurisdiction, that shall not affect:
        1. the validity or enforceability in that jurisdiction of any other
           provision of this License; or
        2. the validity or enforceability in other jurisdictions of that or
           any other provision of this License.

****************************************************************************/

#include <stddef.h>

#include "asciiparse.h"

static int ParseAsciiNum(char cChar, uint32_t *puiNum_p)
{
    if ((cChar >= '0') && (cChar <= '9'))
    {
        *puiNum_p = cChar - '0';
    }
    else if ((cChar >= 'a') && (cChar <= 'f'))
    {
        *puiNum_p = (cChar - 'a') + 10;
    }
    else if ((cChar >= 'A') && (cChar <= 'F'))
    {
        *puiNum_p = (cChar - 'A') + 10;
    }
    else
    {
        return 1;
    }
    return 0;
}

static int ParseAsciiCanData(const unsigned char* pszBuf_p, uint8_t *pCanData_p, const unsigned char** ppEndBuf_p)
{
    size_t i;
    uint32_t uiNum, uiShift;
    uint8_t ubData;

    ubData = 0;
    //0x
    if ((pszBuf_p[0] != '0') || ((pszBuf_p[1] != 'x') && (pszBuf_p[1] != 'X')))
    {
        *ppEndBuf_p = pszBuf_p += 2;
        return 1;
    }
    *ppEndBuf_p = pszBuf_p += 2;

    //00
    for (i = 0; i < 2; ++i)
    {
        if (ParseAsciiNum(pszBuf_p[0], &uiNum))
        {
            pszBuf_p += 1;
            *ppEndBuf_p = pszBuf_p;
            return 1;
        }
        // Shift is multiple of 4 to generate the exponent at base of 16
        uiShift = (4 * (2 - i - 1));
        ubData += uiNum << uiShift;
        pszBuf_p += 1;
    }

    //;
    if (pszBuf_p[0] != ';')
    {
        *ppEndBuf_p = pszBuf_p + 1;
        return 1;
    }
    *pCanData_p = ubData;
    *ppEndBuf_p = pszBuf_p + 1;
    return 0;
}

static int ParseAsciiCanID(const unsigned char* pszBuf_p, struct AsciiCanFrame *pFrame_p, const unsigned char** ppEndBuf_p)
{
    unsigned int uiNum, uiShift;
    size_t i;
    uint32_t uiCanId;

    uiCanId = 0;
    //0x
    if ((pszBuf_p[0] != '0') || ((pszBuf_p[1] != 'x') && (pszBuf_p[1] != 'X')))
    {
        *ppEndBuf_p = pszBuf_p += 2;
        return 1;
    }
    pszBuf_p += 2;
    if (pFrame_p->fExt == 0)
    {
        //7FF
        for (i = 0; i < 3; ++i)
        {
            if (ParseAsciiNum(pszBuf_p[0], &uiNum))
            {
                pszBuf_p += 1;
                *ppEndBuf_p = pszBuf_p;
                return 1;
            }
            // Shift is multiple of 4 to generate the exponent at base of 16
            uiShift = (4 * (3 - i - 1));
            uiCanId += uiNum << uiShift;
            pszBuf_p += 1;
        }
        pFrame_p->uiCanId = uiCanId;
        if (pFrame_p->uiCanId > 0x7ff)
        {
            return 1;
        }
    }
    else
    {
        //1FFFFFFF
        for (i = 0; i < 8; ++i)
        {
            if (ParseAsciiNum(pszBuf_p[0], &uiNum))
            {
                pszBuf_p += 1;
                *ppEndBuf_p = pszBuf_p;
                return 1;
            }
            // Shift is multiple of 4 to generate the exponent at base of 16
            uiShift = (4 * (8 - i - 1));
            uiCanId += uiNum << uiShift;
            pszBuf_p += 1;
        }
        pFrame_p->uiCanId = uiCanId;
        if (pFrame_p->uiCanId > 0x1FFFFFFF)
        {
            return 1;
        }
    }

    //;
    if (pszBuf_p[0] != ';')
    {
        *ppEndBuf_p = pszBuf_p + 1;
        return 1;
    }
    *ppEndBuf_p = pszBuf_p + 1;
    return 0;
}

int ParseAsciiFrame(const unsigned char* pszBuf_p, struct AsciiCanFrame *pFrame_p, const unsigned char** ppEndBuf_p)
{
    const unsigned char* pBuf = pszBuf_p;
    const unsigned char* pEndBuf = pszBuf_p;
    size_t i;

    if (ppEndBuf_p == NULL)
    {
        return 1;
    }

    if (pszBuf_p == NULL)
    {
        *ppEndBuf_p = NULL;
        return 2;
    }

    if (pFrame_p == NULL)
    {
        *ppEndBuf_p = pszBuf_p;
        return 3;
    }

    /* Skip any whitespaces */
    while ((pBuf[0] == '\n')
            || (pBuf[0] == '\r')
            || (pBuf[0] == '\t')
            || (pBuf[0] == ' '))
    {
        pBuf++;
    }

    //$CAN;
    if ((pBuf[0] != '$')
        || (pBuf[1] != 'C')
        || (pBuf[2] != 'A')
        || (pBuf[3] != 'N')
        || (pBuf[4] != ';')
    )
    {
        *ppEndBuf_p = &pBuf[5];
        return 4;
    }
    pBuf += 5;

    //S;
    switch (pBuf[0])
    {
        case 'S':
            pFrame_p->fExt = 0;
            break;
        case 'E':
            pFrame_p->fExt = 1;
            break;
        default:
            *ppEndBuf_p = pBuf + 1;
            return 5;
    }
    if (pBuf[1] != ';')
    {
        *ppEndBuf_p = pBuf + 2;
        return 6;
    }
    pBuf += 2;

    //D;
    switch (pBuf[0])
    {
        case 'D':
            pFrame_p->fRtr = 0;
            break;
        case 'R':
            pFrame_p->fRtr = 1;
            break;
        default:
            *ppEndBuf_p = pBuf + 1;
            return 7;
    }
    if (pBuf[1] != ';')
    {
        *ppEndBuf_p = pBuf + 2;
        return 8;
    }
    pBuf += 2;

    //0x123;
    //0x1fffffff;
    if (ParseAsciiCanID(pBuf, pFrame_p, &pEndBuf) != 0)
    {
        *ppEndBuf_p = pEndBuf;
        return 9;
    }
    pBuf = pEndBuf;

    //0x
    if ((pBuf[0] != '0') || ((pBuf[1] != 'x') && (pBuf[1] != 'X')))
    {
        *ppEndBuf_p = pBuf += 2;
        return 10;
    }
    pBuf += 2;
    //8
    if (ParseAsciiNum(pBuf[0], &pFrame_p->uiCanDlc))
    {
        *ppEndBuf_p = pBuf + 1;
        return 11;
    }
    if (pFrame_p->uiCanDlc > 8)
    {
        *ppEndBuf_p = pBuf + 1;
        return 12;
    }

    if (pBuf[1] != ';')
    {
        *ppEndBuf_p = pBuf + 2;
        return 13;
    }
    pBuf += 2;

    //Only if no RTR-Frame
    if (!pFrame_p->fRtr)
    {
        //0x00;0x01;0x02;0x03;0x04;0x05;0x06;0x07;
        for (i = 0; i < pFrame_p->uiCanDlc; ++i)
        {
            if (ParseAsciiCanData(&pBuf[0], &pFrame_p->abData[i], &pEndBuf))
            {
                *ppEndBuf_p = pEndBuf;
                return 14;
            }
            pBuf = pEndBuf;
        }
    }

    //#
    if (pBuf[0] != '#')
    {
        *ppEndBuf_p = pBuf + 1;
        return 15;
    }
    *ppEndBuf_p = pBuf + 1;
    return 0;
}
