/****************************************************************************

  (c) SYSTEC electronic GmbH, D-08468 Heinsdorfergrund, Am Windrad 2
      www.systec-electronic.com

  Project:      asciiparse library

  Description:  declaration for library

  License:

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

    3. Neither the name of SYSTEC electronic GmbH nor the names of its
       contributors may be used to endorse or promote products derived
       from this software without prior written permission. For written
       permission, please contact info@systec-electronic.com.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.

    Severability Clause:

        If a provision of this License is or becomes illegal, invalid or
        unenforceable in any jurisdiction, that shall not affect:
        1. the validity or enforceability in that jurisdiction of any other
           provision of this License; or
        2. the validity or enforceability in other jurisdictions of that or
           any other provision of this License.

****************************************************************************/

/**
    @file    asciiparse.h
    @author  A. Stein
    @brief   Parser for ASCII CAN messages.
*/
#ifndef _ASCIIPARSE_H
#define _ASCIIPARSE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

struct AsciiCanFrame {
    uint32_t    uiCanId;         /**< 32 bit CAN_ID */
    uint32_t    uiCanDlc;        /**< Frame payload length in Byte (0 .. 8) */
    uint8_t     abData[8];        /**< Payload */
    uint8_t     fRtr;            /**< Flag indicating this is a RTR frame */
    uint8_t     fExt;            /**< Flag indicating this is a frame with extended ID */
};

/**
 * \brief Parses the string pointed to by pszBuf_p
 *
 * \param pszBuf_p pointer to string which is parsed
 * \param pFrame_p pointer to can frame which is filled with parsed data
 * \param ppEndBuf_p pointer to byte after parsed data
 * \retval 0 if successful
 * \retval 1 ppEndBuf_p is NULL
 * \retval 2 pszBuf_p is NULL
 * \retval 3 pFrame_p is NULL
 * \retval 4 the header '$CAN;' does not match
 * \retval 5 the CAN ID type is neither 'S' nor 'E'
 * \retval 6 ';' after CAN ID type is missing
 * \retval 7 the CAN data type is neither 'D' nor 'R'
 * \retval 8 ';' after CAN data type is missing
 * \retval 9 CAN ID could not be parsed
 * \retval 10 prefix '0x' for DLC is wrong
 * \retval 11 DLC number could not be parsed
 * \retval 12 DLC is greater than 8
 * \retval 13 ';' after CAN ID is missing
 * \retval 14 CAN data could not be parsed
 * \retval 15 end character '#' is missing
 * 
 * This function parsed the string pointed to by pszBuf_p. It expects the fixed ASCII format
 * e.g. \verbatim $CAN;S;D;0x123;8;0x00;0x01;0x02;0x03;0x04;0x05;0x06;0x07;# \endverbatim Any whitespaces before
 * this string are skipped.
 * In case of an error the pointer passed per ppEndBuf_p is set to the next character after the error was detected.
 */
int ParseAsciiFrame(const unsigned char* pszBuf_p, struct AsciiCanFrame *pFrame_p, const unsigned char** ppEndBuf_p);

#ifdef __cplusplus
}
#endif

#endif
